/**
 * @file libpackman/db_api.h Database Access API
 * 
 * $Id: db_api.h,v 1.29 2002/04/20 07:38:08 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_DB_API_H_
#define _LIBPACKMAN_DB_API_H_

#include <libpackman/types.h>
#include <libpackman/db.h>

#ifdef __cplusplus
extern "C" {
#endif

/**************************************************************************/
/** @name Database Setup Functions                                        */
/**************************************************************************/
/*@{*/

/**
 * Opens a database of the specified type.
 *
 * The @a type parameter must match one of the installed package modules.
 * These are located in @a /usr/lib/packman/database,
 * @a /usr/local/lib/packman/database, and/or
 * @a $HOME/.gnupdate/packman/database.
 *
 * @param type The type of database.
 * @param mode The access mode.
 *
 * @return A PmDatabase structure representing the opened database, or NULL
 *         if unsuccessful.
 */
PmDatabase *pmOpenDatabase(const char *type, PmAccessMode mode);

/**
 * Creates a new database of the specified type.
 *
 * The @a type parameter must match one of the installed package modules.
 * These are located in @a /usr/lib/packman/database,
 * @a /usr/local/lib/packman/database, and/or
 * @a $HOME/.gnupdate/packman/database.
 *
 * If successful, the PmDatabase structure returned will be a handle to
 * the newly created and opened database. The database will be opened
 * in read/write mode.
 *
 * @param type The type of database.
 * 
 * @return A PmDatabase structure representing the created database,
 *         or NULL if unsuccessful.
 */
PmDatabase *pmCreateDatabase(const char *type);

/**
 * Closes the specified database.
 *
 * This will free up all memory associated with the database, except for
 * any existing PmPackage structures still in memory.
 *
 * All PmPackage structures returned from a call to the database
 * should be destroyed before pmCloseDatabase() is called.
 *
 * @param db The database to close.
 */
void pmCloseDatabase(PmDatabase *db);

/**
 * Rebuilds the database of the specified type.
 *
 * The @a type parameter must match one of the installed package modules.
 * These are located in @a /usr/lib/packman/database,
 * @a /usr/local/lib/packman/database, and/or
 * @a $HOME/.gnupdate/packman/database.
 *
 * @note
 * The database should be closed before you attempt to rebuild it.
 *
 * @param type The type of database.
 *
 * @return The status of the operation.
 */
PmStatus pmRebuildDatabase(const char *type);

/*@}*/


/**************************************************************************/
/** @name PmMatches (De)allocation Functions                              */
/**************************************************************************/
/*@{*/

/**
 * Returns the number of packages in the database.
 *
 * @param db The database.
 *
 * @return The number of packages in the database.
 */
unsigned long pmDbGetPackageCount(PmDatabase *db);

/*@}*/


/**************************************************************************/
/** @name PmMatches (De)allocation Functions                              */
/**************************************************************************/
/*@{*/

/**
 * Creates a new PmMatches structure.
 *
 * @param db The database associated with these matches.
 *
 * @return A new PmMatches structure.
 */
PmMatches *pmNewMatches(PmDatabase *db);

/**
 * Destroys a PmMatches structure.
 *
 * @param matches The structure to destroy.
 */
void pmDestroyMatches(PmMatches *matches);

/*@}*/


/**************************************************************************/
/** @name Database Manipulation Functions                                 */
/**************************************************************************/
/*@{*/

/**
 * Installs a package and adds it to the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to add the package to.
 * @param package     The package to install.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmUninstallPackage()
 */
PmStatus pmInstallPackage(PmDatabase *db, PmPackage *package, char checkDeps,
						  PmTransactionError **error);

/**
 * Uninstalls a package and removes it from the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 * 
 * @param db          The database to add the package to.
 * @param package The package to uninstall.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmInstallPackage()
 */
PmStatus pmUninstallPackage(PmDatabase *db, PmPackage *package, char checkDeps,
							PmTransactionError **error);

/**
 * Installs a batch of packages and adds them to the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to add to.
 * @param batch       The batch to install.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmUninstallBatch()
 */
PmStatus pmInstallBatch(PmDatabase *db, PmBatch *batch, char checkDeps,
						PmTransactionError **error);

/**
 * Uninstalls a batch of packages and removes them the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to remove from.
 * @param batch       The batch to uninstall.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 * 
 * @see pmInstallBatch()
 */
PmStatus pmUninstallBatch(PmDatabase *db, PmBatch *batch, char checkDeps,
						  PmTransactionError **error);

/**
 * Adds the information on a package to the database.
 *
 * This is different from pmInstallPackage(), which adds the information
 * and then proceeds to extract and install the package's files.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to add the package to.
 * @param package     The package to add to the database.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmDbRemovePackage()
 */
PmStatus pmDbAddPackage(PmDatabase *db, PmPackage *package, char checkDeps,
						PmTransactionError **error);

/**
 * Removes the information on a package from the database.
 *
 * This is different from pmUninstallPackage(), which removes the
 * information and then proceeds to uninstall the package's files.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to remove the package from.
 * @param package     The package to remove from the database.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmDbAddPackage()
 */
PmStatus pmDbRemovePackage(PmDatabase *db, PmPackage *package, char checkDeps,
						   PmTransactionError **error);

/**
 * Adds a batch of packages to the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to add to.
 * @param batch       The batch to add.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmDbRemoveBatch()
 */
PmStatus pmDbAddBatch(PmDatabase *db, PmBatch *batch, char checkDeps,
					  PmTransactionError **error);

/**
 * Removes a batch of packages from the database.
 *
 * Under almost all circumstances, you should specify 1 for @a checkDeps.
 *
 * @param db          The database to remove the batch from.
 * @param batch       The batch to remove.
 * @param checkDeps   1 if dependency checking should be enabled. 0 otherwise.
 * @param error       Returned errors, if any. The result of this can be
 *                    NULL if successful, or if the error does not lie in
 *                    anything package-related (i.e., missing database
 *                    function).
 *
 * @return The status of this operation.
 *
 * @see pmDbAddBatch()
 */
PmStatus pmDbRemoveBatch(PmDatabase *db, PmBatch *batch, char checkDeps,
						 PmTransactionError **error);

/*@}*/


/**************************************************************************/
/** @name Search Functions                                                */
/**************************************************************************/
/*@{*/

/**
 * Searches for a package by name.
 *
 * @param db      The database to search.
 * @param name    The name of the package to search for.
 * @param matches The returned PmMatches structure.
 * 
 * @return The status of the search.
 */
PmStatus pmFindByName(PmDatabase *db, const char *name, PmMatches **matches);

/**
 * Searches for a package by group.
 *
 * @param db      The database to search.
 * @param group   The group to search for.
 * @param matches The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmFindByGroup(PmDatabase *db, const char *group, PmMatches **matches);

/**
 * Searches for a package by file.
 *
 * @param db      The database to search.
 * @param file    The file to search for.
 * @param matches The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmFindByFile(PmDatabase *db, const char *file, PmMatches **matches);

/**
 * Searches for a package by the specified provided dependency.
 * 
 * @param db       The database to search.
 * @param provides The provided dependency to search with.
 * @param matches  The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmFindByProvides(PmDatabase *db, const char *provides,
						  PmMatches **matches);

/**
 * Searches for a package by the specified required dependency.
 *
 * @param db       The database to search.
 * @param requires The required dependency to search with.
 * @param matches  The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmFindByRequires(PmDatabase *db, const char *requires,
						  PmMatches **matches);

/**
 * Searches for a package by the specified conflicting dependency.
 *
 * @param db        The database to search.
 * @param conflicts The conflicting dependency to search with.
 * @param matches   The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmFindByConflicts(PmDatabase *db, const char *conflicts,
						   PmMatches **matches);

/**
 * Returns all packages in the database.
 *
 * @param db      The database.
 * @param matches The returned PmMatches structure.
 *
 * @return The status of the search.
 */
PmStatus pmGetAllPackages(PmDatabase *db, PmMatches **matches);

/*@}*/


/**************************************************************************/
/** @name Match Iteration Functions                                       */
/**************************************************************************/
/*@{*/

/**
 * Returns the first matched package.
 *
 * The returned package must be freed.
 *
 * @param matches The matches.
 * 
 * @return The first matched package.
 *
 * @see pmNextMatch()
 */
PmPackage *pmFirstMatch(PmMatches *matches);

/**
 * Returns the next matched package.
 *
 * The returned package must be freed.
 *
 * @param matches The matches.
 *
 * @return The next matched package.
 *
 * @see pmFirstMatch()
 */
PmPackage *pmNextMatch(PmMatches *matches);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_DB_API_H_ */
