/**
 * @file libpackman/dependency.h Dependency API
 *
 * $Id: dependency.h,v 1.12 2002/06/08 04:53:37 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_DEPENDENCY_H_
#define _LIBPACKMAN_DEPENDENCY_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _PmDependency     PmDependency;     /**< PmDependency     */
typedef enum   _PmDependencyType PmDependencyType; /**< PmDependencyType */

#include <libpackman/package.h>


/**
 * A dependency type.
 */
enum _PmDependencyType
{
	PM_DEP_UNKNOWN = 0,  /**< Unknown dependency (usually an error.)       */
	PM_DEP_PROVIDES,     /**< A provided dependency.                       */
	PM_DEP_REQUIRED,     /**< A standard, required dependency.             */
	PM_DEP_PRE_REQUIRED, /**< A dependency that must be installed first.   */
	PM_DEP_RECOMMENDED,  /**< A strongly recommended dependency.           */
	PM_DEP_SUGGESTED,    /**< A suggested, but very optional dependency.   */
	PM_DEP_ENHANCES,     /**< A dependency that Enhances this package.     */
	PM_DEP_REPLACES,     /**< A dependency that gets replaced by this one. */
	PM_DEP_CONFLICTS,    /**< A conflicting dependency.                    */
	PM_DEP_VIRTUAL       /**< A virtual dependency.                        */
};

/**
 * @struct _PmDependency dependency.h libpackman/dependency.h
 *
 * A dependency.
 *
 * This can represent a required, provided, conflicting, or recommended
 * package dependency.
 */
struct _PmDependency
{
	char *name;                  /**< The name of the dependency.          */
	char *owner;                 /**< The name of the dependency's owner.  */
	char *version;               /**< The version required.                */
	PmRelationship versionRel;   /**< Version relationship.                */

	PmDependencyType type;       /**< The dependency type.                 */

	PmDependency *altDeps;       /**< Alternative dependencies.            */
	PmDependency *lastAltDep;    /**< The last alternative dependency.     */
	unsigned int altDepCount;    /**< Number of alternative dependencies.  */

	PmDependency *prev;          /**< The previous dependency in the list. */
	PmDependency *next;          /**< The next dependency in the list.     */
};


/**************************************************************************/
/** @name PmDependency (De)allocation Functions                           */
/**************************************************************************/
/*@{*/

/**
 * Creates a new PmDependency.
 * 
 * @return A new, empty PmDependency.
 *
 * @see pmDestroyDependency()
 * @see pmDuplicateDependency()
 */
PmDependency *pmNewDependency(void);

/**
 * Duplicates an existing PmDependency.
 *
 * @param baseDep The PmDependency to duplicate.
 *
 * @return A new copy of \a baseDep.
 *
 * @see pmNewDependency()
 * @see pmDestroyDependency()
 */
PmDependency *pmDuplicateDependency(PmDependency *baseDep);

/**
 * Destroys the specified PmDependency.
 *
 * @param dependency The PmDependency to destroy.
 *
 * @see pmNewDependency()
 * @see pmDuplicateDependency()
 */
void pmDestroyDependency(PmDependency *dependency);

/*@}*/

/**************************************************************************/
/** @name Attribute Modification Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Sets the name of the specified dependency.
 *
 * @param dep  The dependency to modify.
 * @param name The new name for the dependency.
 *
 * @see pmGetDependencyName()
 */
void pmSetDependencyName(PmDependency *dep, const char *name);

/**
 * Sets the version and the relationship for the dependency.
 *
 * The relationship specifies whether the version needed is
 * <, <=, =, >=, or > the value of @a version.
 *
 * @param dep     The dependency to modify.
 * @param version The version for the dependency.
 * @param rel     The relationship of the version.
 *
 * @see pmGetDependencyVersion()
 * @see PmRelationship
 */
void pmSetDependencyVersion(PmDependency *dep, const char *version,
							PmRelationship rel);

/**
 * Sets the name of the dependency's owner.
 *
 * @param dep   The dependency to modify.
 * @param owner The owner of the dependency.
 *
 * @see pmGetDependencyOwner()
 */
void pmSetDependencyOwner(PmDependency *dep, const char *owner);

/**
 * Sets the dependency's type.
 *
 * @param dep  The dependency.
 * @param type The dependency type.
 *
 * @see pmGetDependencyType()
 */
void pmSetDependencyType(PmDependency *dep, PmDependencyType type);

/*@}*/


/**************************************************************************/
/** @name Attribute Retrieval Functions                                   */
/**************************************************************************/
/*@{*/

/**
 * Returns the name of the specified dependency.
 *
 * @param dep The dependency.
 *
 * @return The dependency's name.
 *
 * @see pmSetDependencyName()
 */
const char *pmGetDependencyName(PmDependency *dep);

/**
 * Returns the version and the relationship of the specified dependency.
 * 
 * @param dep     The dependency.
 * @param version The returned version.
 * @param rel     The returned relationship.
 *
 * @return The dependency's version.
 *
 * @see pmSetDependencyVersion()
 */
const char *pmGetDependencyVersion(PmDependency *dep, const char **version,
								   PmRelationship *rel);

/**
 * Returns the name of the dependency's owner.
 *
 * @param dep The dependency.
 *
 * @return The dependency's owner's name.
 * 
 * @see pmSetDependencyOwner()
 */
const char *pmGetDependencyOwner(PmDependency *dep);

/**
 * Returns the dependency's type.
 *
 * @param dep The dependency.
 *
 * @return The dependency's type.
 *
 * @see pmSetDependencyType()
 */
PmDependencyType pmGetDependencyType(PmDependency *dep);

/*@}*/


/**************************************************************************/
/** @name Requirements Iteration Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Returns the first requirement in the list.
 * 
 * @param package The package containing the requirements.
 *
 * @return The first requirement in the list.
 *
 * @see pmNextRequirement()
 * @see pmPreviousRequirement()
 */
PmDependency *pmFirstRequirement(PmPackage *package);

/**
 * Returns the next requirement in the list.
 *
 * @param dep The current requirement.
 *
 * @return The next requirement in the list.
 *
 * @see pmFirstRequirement()
 * @see pmPreviousRequirement()
 */
PmDependency *pmNextRequirement(PmDependency *dep);

/**
 * Returns the previous requirement in the list.
 *
 * @param dep The previous requirement.
 *
 * @return The previous requirement in the list.
 *
 * @see pmFirstRequirement()
 * @see pmNextRequirement()
 */
PmDependency *pmPreviousRequirement(PmDependency *dep);

/*@}*/


/**************************************************************************/
/** @name Alternative Dependency Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Adds an alternative required dependency to an existing dependency.
 *
 * @param base The base dependency.
 * @param alt  The alternative dependency.
 */
void pmAddAltRequirement(PmDependency *base, PmDependency *alt);

/**
 * Returns the first alternative required dependency.
 * 
 * @param dep The base dependency.
 *
 * @return The first alternative required dependency.
 */
PmDependency *pmFirstAltRequirement(PmDependency *dep);

/**
 * Returns the number of alternative required dependencies for a specified
 * dependency.
 *
 * @param dep The base dependency.
 *
 * @return The number of alternative dependencies, excluding the base
 *         dependency.
 */
unsigned int pmGetAltRequirementCount(PmDependency *dep);

/*@}*/

/**************************************************************************/
/** @name Provisions Iteration Functions                                  */
/**************************************************************************/
/*@{*/

/**
 * Returns the first provide in the list.
 * 
 * @param package The package containing the provides.
 *
 * @return The first provide in the list.
 *
 * @see pmNextProvide()
 * @see pmPreviousProvide()
 */
PmDependency *pmFirstProvide(PmPackage *package);

/**
 * Returns the next provide in the list.
 *
 * @param dep The current provide.
 *
 * @return The next provide in the list.
 *
 * @see pmFirstProvide()
 * @see pmPreviousProvide()
 */
PmDependency *pmNextProvide(PmDependency *dep);

/**
 * Returns the previous provide in the list.
 *
 * @param dep The previous provide.
 *
 * @return The previous provide in the list.
 *
 * @see pmFirstProvide()
 * @see pmNextProvide()
 */
PmDependency *pmPreviousProvide(PmDependency *dep);

/*@}*/


#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_DEPENDENCY_H_ */
