/**
 * @file libpackman/downloadurl.h Download URL API
 * 
 * $Id: downloadurl.h,v 1.8 2002/03/06 01:36:08 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_DOWNLOADURL_H_
#define _LIBPACKMAN_DOWNLOADURL_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _PmDownloadUrl PmDownloadUrl;  /**< PmDownloadUrl */

#include <libpackman/package.h>

/**
 * @struct _PmDownloadUrl downloadurl.h libpackman/downloadurl.h
 *
 * A download URL.
 *
 * This represents a location where the package can be downloaded, either
 * locally or remotely.
 */
struct _PmDownloadUrl
{
	char *url;               /**< The URL of the package.                */

	PmDownloadUrl *prev;     /**< The previous download URL in the list. */
	PmDownloadUrl *next;     /**< The next download URL in the list.     */
};


/**************************************************************************/
/** @name PmDownloadUrl (De)allocation Functions                          */
/**************************************************************************/
/*@{*/

/**
 * Creates a new PmDownloadUrl.
 *
 * @return A new, empty PmDownloadUrl.
 *
 * @see pmDuplicateDownloadUrl()
 * @see pmDestroyDownloadUrl()
 */
PmDownloadUrl *pmNewDownloadUrl(void);

/**
 * Duplicates an existing PmDownloadUrl.
 *
 * @param baseDownloadUrl The PmDownloadUrl to duplicate.
 *
 * @return A new copy of \a baseDownloadUrl.
 *
 * @see pmNewDownloadUrl()
 * @see pmDestroyDownloadUrl()
 */
PmDownloadUrl *pmDuplicateDownloadUrl(PmDownloadUrl *baseDownloadUrl);

/**
 * Destroys the specified PmDownloadUrl.
 *
 * @param downloadUrl The PmDownloadUrl to destroy.
 *
 * @see pmNewDownloadUrl()
 * @see pmDuplicateDownloadUrl()
 */
void pmDestroyDownloadUrl(PmDownloadUrl *downloadUrl);

/*@}*/


/**************************************************************************/
/** @name Attribute Modification Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Sets the URL on the specified PmDownloadUrl.
 *
 * @param downloadUrl The PmDownloadUrl to modify.
 * @param url         The new URL.
 *
 * @see pmGetDownloadUrl()
 */
void pmSetDownloadUrl(PmDownloadUrl *downloadUrl, const char *url);

/*@}*/


/**************************************************************************/
/** @name Attribute Retrieval Functions                                   */
/**************************************************************************/
/*@{*/

/**
 * Returns the URL of the specified PmDownloadUrl.
 *
 * @param downloadUrl The PmDownloadUrl.
 *
 * @return \a downloadUrl's URL.
 *
 * @see pmSetDownloadUrl()
 */
const char *pmGetDownloadUrl(PmDownloadUrl *downloadUrl);

/*@}*/


/**************************************************************************/
/** @name Iteration Functions                                             */
/**************************************************************************/
/*@{*/

/**
 * Returns the first PmDownloadUrl in the list.
 *
 * @param pkg The package containing the download URLs.
 *
 * @return The first PmDownloadUrl in the list.
 *
 * @see pmPreviousDownloadUrl()
 * @see pmNextDownloadUrl()
 */
PmDownloadUrl *pmFirstDownloadUrl(PmPackage *pkg);

/**
 * Returns the previous PmDownloadUrl in the list.
 * 
 * @param downloadUrl The current PmDownloadUrl.
 *
 * @return The previous PmDownloadUrl in the list.
 *
 * @see pmFirstDownloadUrl()
 * @see pmNextDownloadUrl()
 */
PmDownloadUrl *pmPreviousDownloadUrl(PmDownloadUrl *downloadUrl);

/**
 * Returns the next PmDownloadUrl in the list.
 *
 * @param downloadUrl The current PmDownloadUrl.
 *
 * @return The next PmDownloadUrl in the list.
 *
 * @see pmFirstDownloadUrl()
 * @see pmPreviousDownloadUrl()
 */
PmDownloadUrl *pmNextDownloadUrl(PmDownloadUrl *downloadUrl);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_DOWNLOADURL_H_ */
