/**
 * @file libpackman/file.h File storage API
 * 
 * $Id: file.h,v 1.13 2002/03/20 07:05:01 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_FILE_H_
#define _LIBPACKMAN_FILE_H_

#include <sys/types.h>
#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _PmFile PmFile;   /**< PmFile */

#include <libpackman/package.h>

/**
 * A file type.
 *
 * @see PmFile
 */
typedef enum _PmFileType
{
	PM_FILETYPE_NORMAL,          /**< A normal file.        */
	PM_FILETYPE_CONFIG,          /**< A configuration file. */
	PM_FILETYPE_DOCUMENTATION,   /**< A documentation file. */
	PM_FILETYPE_DEVICE,          /**< A device file.        */
	PM_FILETYPE_DIRECTORY,       /**< A directory.          */
	PM_FILETYPE_UNKNOWN          /**< An unknown file type. */

} PmFileType;


/**
 * @struct _PmFile file.h libpackman/file.h
 *
 * A representation of a file on the filesystem.
 */
struct _PmFile
{
	PmFileType type;             /**< The type of file.                  */

	PmPackage *package;          /**< The package that owns this file.   */
	
	char   *name;                /**< The filename.                      */
	char   *checksum;            /**< The file's MD5 checksum.           */
	mode_t  mode;                /**< The file's mode.                   */
	char   *owner;               /**< The file's owner username.         */
	char   *group;               /**< The file's owner group name.       */
	char   *symlink;             /**< The file that this file points to. */

	unsigned short majorMinor;   /**< The major and minor numbers for
								  **  this file if it is a device.       */
	
	unsigned int size;           /**< The file size.                     */
	
	time_t date;                 /**< The timestamp in seconds since the
								  **  epoch.                             */
	
	PmFile *prev;                /**< The previous file in the list.     */
	PmFile *next;                /**< The next file in the list.         */
};


/**************************************************************************/
/** @name Structure (De)allocation Functions                              */
/**************************************************************************/
/*@{*/

/**
 * Creates a new PmFile structure.
 *
 * @return A new, empty PmFile structure.
 *
 * @see pmDuplicateFile()
 * @see pmDestroyFile()
 */
PmFile *pmNewFile(void);

/**
 * Duplicates a PmFile structure.
 *
 * @param baseFile The PmFile to duplicate.
 *
 * @return The new duplicate.
 *
 * @see pmNewFile()
 * @see pmDestroyFile()
 */
PmFile *pmDuplicateFile(PmFile *baseFile);

/**
 * Destroys a PmFile structure.
 *
 * @param file The PmFile to destroy.
 *
 * @see pmNewFile()
 * @see pmDuplicateFile()
 */
void pmDestroyFile(PmFile *file);

/*@}*/


/**************************************************************************/
/** @name Attribute Modification Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Sets the package that owns the specified file.
 *
 * This should only be used by libpackman.
 * 
 * @param file    The file.
 * @param package The package that owns this file.
 *
 * @see pmGetFilePackage()
 */
void pmSetFilePackage(PmFile *file, PmPackage *package);

/**
 * Sets the name of the specified file.
 *
 * @param file The file.
 * @param name The name of the file.
 *
 * @see pmGetFileName()
 */
void pmSetFileName(PmFile *file, const char *name);

/**
 * Sets the MD5 checksum of the specified file.
 *
 * @param file     The file.
 * @param checksum The MD5 checksum.
 *
 * @see pmGetFileChecksum()
 */
void pmSetFileChecksum(PmFile *file, const char *checksum);

/**
 * Sets the mode of the specified file.
 *
 * @param file The file.
 * @param mode The file's mode.
 *
 * @see pmGetFileMode()
 */
void pmSetFileMode(PmFile *file, mode_t mode);

/**
 * Sets the owner of the specified file.
 *
 * @param file  The file.
 * @param owner The owner username.
 *
 * @see pmGetFileOwner()
 */
void pmSetFileOwner(PmFile *file, const char *owner);

/**
 * Sets the group of the specified file.
 *
 * @param file  The file.
 * @param group The owner group.
 *
 * @see pmGetFileGroup()
 */
void pmSetFileGroup(PmFile *file, const char *group);

/**
 * Sets the file type of the specified file.
 *
 * @param file The file.
 * @param type The file type.
 *
 * @see PmFileType, pmGetFileType()
 */
void pmSetFileType(PmFile *file, PmFileType type);

/**
 * Sets the major/minor numbers of the specified file.
 * 
 * This is only used if the file is of type PM_FILETYPE_DEVICE.
 * 
 * @param file       The file.
 * @param majorMinor The major/minor numbers.
 *
 * @see pmGetFileMajorMinor()
 */
void pmSetFileMajorMinor(PmFile *file, unsigned short majorMinor);

/**
 * Sets the size of the specified file.
 *
 * @param file The file.
 * @param size The file size.
 *
 * @see pmGetFileSize()
 */
void pmSetFileSize(PmFile *file, unsigned int size);

/**
 * Sets the timestamp of the specified file.
 *
 * @param file The file.
 * @param date The date/time timestamp in seconds since the epoch.
 *
 * @see pmGetFileDate()
 */
void pmSetFileDate(PmFile *file, time_t date);

/**
 * Sets the file that this file links to.
 * 
 * If this file is not a symlink, \a symlink should be \c NULL.
 *
 * @param file    The file.
 * @param symlink The file that this file points to.
 *
 * @see pmGetFileSymlink()
 */
void pmSetFileSymlink(PmFile *file, const char *symlink);

/*@}*/


/**************************************************************************/
/** @name Attribute Retrieval Functions                                   */
/**************************************************************************/
/*@{*/

/**
 * Returns the package that owns the specified file.
 *
 * @param file The file.
 *
 * @return The package that owns this file.
 *
 * @see pmSetFilePackage()
 */
PmPackage *pmGetFilePackage(PmFile *file);

/**
 * Returns the name of the specified file.
 *
 * @param file The file.
 *
 * @return The file's name.
 *
 * @see pmSetFileName()
 */
const char *pmGetFileName(PmFile *file);

/**
 * Returns the MD5 checksum of the specified file.
 *
 * @param file The file.
 *
 * @return The file's MD5 checksum.
 *
 * @see pmSetFileChecksum()
 */
const char *pmGetFileChecksum(PmFile *file);

/**
 * Returns the mode of the specified file.
 *
 * @param file The file.
 * 
 * @return The file's mode.
 * 
 * @see pmSetFileMode()
 */
mode_t pmGetFileMode(PmFile *file);

/**
 * Returns the owner username of the specified file.
 *
 * @param file The file.
 *
 * @return The file's owner username.
 *
 * @see pmSetFileOwner()
 */
const char *pmGetFileOwner(PmFile *file);

/**
 * Returns the owner group of the specified file.
 *
 * @param file The file.
 *
 * @return The file's owner group.
 *
 * @see pmSetFileGroup()
 */
const char *pmGetFileGroup(PmFile *file);

/**
 * Returns the type of the specified file.
 *
 * @param file The file
 *
 * @return The file's type.
 *
 * @see PmFileType, pmSetFileType()
 */
PmFileType pmGetFileType(PmFile *file);

/**
 * Returns the major/minor numbers for this file.
 *
 * @param file The file.
 *
 * @return The file's major/minor device numbers.
 *
 * @see pmSetFileMajorMinor()
 */
unsigned short pmGetFileMajorMinor(PmFile *file);

/**
 * Returns the size of this file.
 *
 * @param file The file.
 *
 * @return The file's size.
 *
 * @see pmSetFileSize()
 */
unsigned int pmGetFileSize(PmFile *file);

/**
 * Returns the date/time in seconds since the epoch.
 *
 * @param file The file.
 *
 * @return The date/time in seconds since the epoch.
 *
 * @see pmSetFileDate()
 */
time_t pmGetFileDate(PmFile *file);

/**
 * Returns the file this file is pointing to (if any).
 *
 * @param file The file.
 *
 * @return A file or directory path if this is a symlink. @c NULL otherwise.
 *
 * @see pmSetFileSymlink()
 */
const char *pmGetFileSymlink(PmFile *file);

/*@}*/


/**************************************************************************/
/** @name Iteration Functions                                             */
/**************************************************************************/
/*@{*/

/**
 * Returns the first file in the package.
 *
 * @param pkg The package.
 *
 * @return The first file in the package.
 *
 * @see pmPreviousFile()
 * @see pmNextFile()
 */
PmFile *pmFirstFile(PmPackage *pkg);

/**
 * Returns the previous file in the list of files.
 *
 * @param file The current file.
 *
 * @return The previous file in the list.
 *
 * @see pmFirstFile()
 * @see pmNextFile()
 */
PmFile *pmPreviousFile(PmFile *file);

/**
 * Returns the next file in the list of files.
 *
 * @param file The current file.
 *
 * @return The next file in the list.
 *
 * @see pmFirstFile()
 * @see pmPreviousFile()
 */
PmFile *pmNextFile(PmFile *file);

/*@}*/

/**************************************************************************/
/** @name Utility Functions                                               */
/**************************************************************************/
/*@{*/

/**
 * Converts a file type to its string equivalent.
 *
 * @param type The file type to convert.
 *
 * @return The string equivalent of the file type.
 *
 * @see PmFileType, pmFileTypeFromName()
 */
const char *pmFileTypeToName(PmFileType type);

/**
 * Converts the string equivalent of a file type to its PmFileType version.
 *
 * @param name The string equivalent of the file type.
 *
 * @return The PmFileType version.
 *
 * @see PmFileType, pmFileTypeToName()
 */
PmFileType pmFileTypeFromName(const char *name);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_FILE_H_ */
