/**
 * @file libpackman/graph.h Graph functions
 *
 * $Id: graph.h,v 1.4 2002/03/20 07:05:01 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_GRAPH_H_
#define _LIBPACKMAN_GRAPH_H_

typedef struct _PmEdge PmEdge;        /**< Graph edge.    */
typedef struct _PmVertice PmVertice;  /**< Graph vertice. */

/**
 * An edge in a graph.
 *
 * Edges connect vertices to each other.
 *
 * @see PmGraph
 * @see PmVertice
 */
struct _PmEdge
{
	PmVertice *target;  /**< The vertice this edge points to. */

	PmEdge *prev;       /**< The previous edge in the list.   */
	PmEdge *next;       /**< The next edge in the list.       */
};

/**
 * A vertice in a graph.
 *
 * Vertices contain data and are connected to each other through edges.
 *
 * @see PmEdge
 * @see PmGraph
 */
struct _PmVertice
{
	void *data;       /**< The data stored in the vertice.               */

	size_t edgeCount; /**< The number of edges coming from this vertice. */

	char visited;     /**< 1 if this vertice has been visited by tsort.  */

	PmEdge *edges;    /**< The first edge in the list.                   */
	PmEdge *lastEdge; /**< The last edge in the list.                    */

	PmVertice *prev;  /**< The previous vertice in the list.             */
	PmVertice *next;  /**< The next vertice in the list.                 */
};

/**
 * A graph, containing vertices and edges.
 *
 * @see PmEdge
 * @see PmVertice
 */
typedef struct
{
	size_t verticeCount;     /**< The number of verticies.       */

	PmVertice *vertices;     /**< The first vertice in the list. */
	PmVertice *lastVertice;  /**< The last vertice in the list.  */

} PmGraph;

/**
 * Creates a new graph.
 *
 * @return A PmGraph structure.
 *
 * @see pmDestroyGraph()
 */
PmGraph *pmNewGraph(void);

/**
 * Destroys a graph and all associated vertices and edges.
 * 
 * The data stored inside verticies will not be deleted.
 *
 * @param graph The graph to destroy.
 *
 * @see pmNewGraph()
 */
void pmDestroyGraph(PmGraph *graph);

/**
 * Creates a new vertice.
 *
 * @param data The data to store in the vertice.
 *
 * @return The new vertice.
 *
 * @see pmDestroyVertice()
 */
PmVertice *pmNewVertice(void *data);

/**
 * Destroys a vertice.
 *
 * This will not destroy the data inside the vertice.
 *
 * @param vertice The vertice to destroy.
 *
 * @see pmNewVertice()
 */
void pmDestroyVertice(PmVertice *vertice);

/**
 * Creates a new edge.
 *
 * @param target The vertice being pointed to.
 *
 * @return The new edge.
 * 
 * @see pmDestroyEdge()
 */
PmEdge *pmNewEdge(PmVertice *target);

/**
 * Destroys an edge.
 *
 * @param edge The edge to destroy.
 *
 * @see pmNewEdge()
 */
void pmDestroyEdge(PmEdge *edge);

/**
 * Adds a vertice to the graph with the specified data.
 *
 * @param graph The graph to add to.
 * @param data  The data to add.
 *
 * @return The newly added vertice.
 *
 * @see pmAddEdge()
 * @see pmNewVertice()
 * @see pmDestroyVertice()
 */
PmVertice *pmAddVertice(PmGraph *graph, void *data);

/**
 * Adds an edge to the graph with the specified vertice as a target.
 *
 * @param parent The parent vertice.
 * @param target The target vertice.
 *
 * @return The newly added edge.
 *
 * @see pmAddVertice()
 * @see pmNewEdge()
 * @see pmDestroyEdge()
 */
PmEdge *pmAddEdge(PmVertice *parent, PmVertice *target);

/**
 * Builds a dependency graph from the specified batch of packages.
 *
 * @param batch The batch of packages.
 *
 * @return The graph of dependencies.
 *
 * @see pmNewGraph()
 * @see pmDestroyGraph()
 * @see pmTSortPackages()
 */
PmGraph *pmBuildDepGraph(PmBatch *batch);

/**
 * Performs a topological sort on a graph containing packages.
 *
 * @param graph The graph to perform the sort on.
 * @param batch The batch to put the packages in.
 *
 * @see pmBuildDepGraph()
 */
void pmTSortPackages(PmGraph *graph, PmBatch *batch);

#endif /* _LIBPACKMAN_GRAPH_H_ */
