/**
 * @file libpackman/hashtable.h Hash Table functions
 *
 * $Id: hashtable.h,v 1.3 2002/05/30 06:43:35 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_HASHTABLE_H_
#define _LIBPACKMAN_HASHTABLE_H_

typedef struct _PmHashTable PmHashTable; /**< Hash table. */
typedef struct _PmBucket    PmBucket;    /**< Bucket.     */

/**
 * A hash table.
 */
struct _PmHashTable
{
	unsigned char  bucketCount; /**< The number of buckets.              */
	unsigned short itemCount;   /**< The number of items.                */

	PmBucket     **buckets;     /**< The buckets.                        */
	unsigned char *counts;      /**< The number of items in each bucket. */

	/**
	 * The key comparison function.
	 *
	 * @param key1 The first key.
	 * @param key2 The second key.
	 *
	 * @return -1 if @a key1 is less than @a key2.
	 * @return  0 if @a key1 equals @a key2.
	 * @return  1 if @a key1 is greater than @a key2.
	 */
	int (*compare)(const void *key1, const void *key2);

	/**
	 * Generates a hash index for the specified key.
	 *
	 * @param table The hash table.
	 * @param key   The key.
	 *
	 * @return The hash index.
	 */
	unsigned short (*hashkey)(PmHashTable *table, const void *key);

	/**
	 * The memory deallocation function for the data.
	 *
	 * @param data The data to deallocate.
	 */
	void (*freeData)(void *data);
};

/**
 * A bucket in a hash table.
 *
 * @see PmHashTable
 */
struct _PmBucket
{
	void *key;          /**< The key.         */
	void *data;         /**< The data.        */

	PmBucket *next;     /**< The next bucket. */
};

/**
 * Creates a new hash table.
 *
 * @param compare  The key comparison function.
 * @param hashkey  The hash key generation function.
 * @param freeData The memory deallocation function for data
 *                 (or @c NULL to not deallocate automatically).
 *
 * @return A new hash table.
 *
 * @see pmDestroyHashTable()
 */
PmHashTable *pmNewHashTable(
	int (*compare)(const void *key1, const void *key2),
	unsigned short (*hashkey)(PmHashTable *table, const void *key),
	void (*freeData)(void *data));

/**
 * Destroys a hash table.
 *
 * @param table The hash table to destroy.
 *
 * @see pmNewHashTable()
 */
void pmDestroyHashTable(PmHashTable *table);

/**
 * Adds a key and its associated data to a hash table.
 *
 * @param table The hash table.
 * @param key   The key to add.
 * @param data  The data to add.
 *
 * @return 1 on success. 0 on conflict or bad parameter.
 *
 * @see pmHashTableRemove()
 * @see pmHashTableGet()
 */
char pmHashTableAdd(PmHashTable *table, void *key, void *data);

/**
 * Removes a key and its associated data from a hash table.
 *
 * @param table The hash table.
 * @param key   The key.
 *
 * @return 1 on success. 0 on bad parameter, or if @a key is not found.
 *
 * @see pmHashTableAdd()
 * @see pmHashTableGet()
 */
char pmHashTableRemove(PmHashTable *table, void *key);

/**
 * Returns the data with the specified key.
 *
 * @param table The hash table.
 * @param key   The key the data is associated with.
 *
 * @return The data, or @c NULL if not found.
 *
 * @see pmHashTableAdd()
 * @see pmHashTableRemove()
 */
void *pmHashTableGet(PmHashTable *table, void *key);

#endif /* _LIBPACKMAN_HASHTABLE_H_ */

