/**
 * @file libpackman/module.h Module API
 * 
 * $Id: module.h,v 1.38 2002/06/05 17:02:29 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_MODULE_H_
#define _LIBPACKMAN_MODULE_H_

#ifdef __cplusplus
extern "C" {
#endif

typedef enum   _PmModuleType  PmModuleType;   /**< Module types.        */
typedef struct _PmDatabaseOps PmDatabaseOps;  /**< Database operations. */
typedef struct _PmPackageOps  PmPackageOps;   /**< Package operations.  */
typedef struct _PmModule      PmModule;       /**< A loadable module.   */

#include <libpackman/package.h>
#include <libpackman/db.h>
#include <libpackman/script.h>

/**
 * Handles the initialization of modules.
 */
#ifdef STATIC_MODULE
# define PM_INIT_DB_MODULE(modname, initfunc, dbops) \
	PmModule *init_##modname##_db_module(void) { \
		initfunc(PM_MODULE_DATABASE); \
		return pmRegisterModule(#modname, &(dbops), PM_MODULE_DATABASE); \
	}
# define PM_INIT_PKG_MODULE(modname, initfunc, pkgops) \
	PmModule *init_##modname##_pkg_module(void) { \
		initfunc(PM_MODULE_PACKAGE); \
		return pmRegisterModule(#modname, &(pkgops), PM_MODULE_PACKAGE); \
	}
#else /* if !STATIC_MODULE */
# define PM_INIT_DB_MODULE(modname, initfunc, dbops) \
	PmModule *initPackmanModule(void) { \
		initfunc(PM_MODULE_DATABASE); \
		return pmRegisterModule(#modname, &(dbops), PM_MODULE_DATABASE); \
	}
# define PM_INIT_PKG_MODULE(modname, initfunc, pkgops) \
	PmModule *initPackmanModule(void) { \
		initfunc(PM_MODULE_PACKAGE); \
		return pmRegisterModule(#modname, &(pkgops), PM_MODULE_PACKAGE); \
	}
#endif

/**
 * A module type.
 *
 * These are the types of modules that libpackman can dynamically load.
 */
enum _PmModuleType
{
	PM_MODULE_PACKAGE,  /**< Package handler module. */
	PM_MODULE_DATABASE  /**< Database access module. */
};

/**
 * @struct _PmDatabaseOps module.h libpackman/module.h
 *
 * Database operations structure.
 *
 * These functions should only be used by libpackman!
 */
struct _PmDatabaseOps
{
	/**
	 * Opens the database.
	 *
	 * @param db The database to open.
	 * 
	 * @return The status of the operation.
	 *
	 * @see pmOpenDatabase()
	 */
	PmStatus (*open)(PmDatabase *db);

	/**
	 * Creates a new database.
	 *
	 * @param db The database handle.
	 *
	 * @return The status of the operation.
	 *
	 * @see pmCreateDatabase()
	 */
	PmStatus (*create)(PmDatabase *db);

	/**
	 * Closes the database.
	 *
	 * @param db The database to close.
	 *
	 * @return The status of the operation.
	 *
	 * @see pmCloseDatabase()
	 */
	PmStatus (*close)(PmDatabase *db);

	/**
	 * Rebuilds the database.
	 *
	 * @param db The database to rebuild.
	 *
	 * @return The status of the operation.
	 *
	 * @see pmRebuildDatabase()
	 */
	PmStatus (*rebuild)(PmDatabase *db);

	/**
	 * Returns the number of packages in the database.
	 *
	 * @param db The database.
	 *
	 * @return The number of packages.
	 *
	 * @see pmDatabaseGetPackageCount()
	 */
	unsigned long (*getPackageCount)(PmDatabase *db);

	/**
	 * Adds the information on a package to the database.
	 *
	 * This is different from pmInstallPackage(), which adds the information
	 * and then proceeds to extract and install the package's files.
	 *
	 * @param db  The database to add the package to.
	 * @param pkg The package to add to the database.
	 * 
	 * @return The status of the operation.
	 *
	 * @see pmDatabaseAddPackage()
	 */
	PmStatus (*addPackage)(PmDatabase *db, PmPackage *pkg);
	
	/**
	 * Removes the information on a package from the database.
	 *
	 * This is different from pmUninstallPackage(), which removes
	 * the information and then proceeds to uninstall the package's files.
	 *
	 * @param db      The database to remove the package from.
	 * @param package The package to remove.
	 *
	 * @return The status of this operation.
	 *
	 * @see pmDatabaseRemovePackage()
	 */
	PmStatus (*removePackage)(PmDatabase *db, PmPackage *package);
	
	/**
	 * Searches for a package by name.
	 *
	 * @param db      The database to search.
	 * @param name    The name of the package to search for.
	 * @param matches The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmFindByName()
	 */
	PmStatus (*findByName)(PmDatabase *db, const char *name,
						   PmMatches *matches);

	/**
	 * Searches for a package by group.
	 *
	 * @param db      The database to search.
	 * @param group   The group to search for.
	 * @param matches The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmFindByGroup()
	 */
	PmStatus (*findByGroup)(PmDatabase *db, const char *group,
							PmMatches *matches);

	/**
	 * Searches for a package by file.
	 *
	 * @param db      The database to search.
	 * @param file    The file to search for.
	 * @param matches The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmFindByFile()
	 */
	PmStatus (*findByFile)(PmDatabase *db, const char *file,
						   PmMatches *matches);

	/**
	 * Searches for a package by the specified required dependency.
	 *
	 * @param db       The database to search.
	 * @param requires The required dependency to search with.
	 * @param matches  The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmFindByRequires()
	 */
	PmStatus (*findByRequires)(PmDatabase *db, const char *requires,
							   PmMatches *matches);

	/**
	 * Searches for a package by the specified provided dependency.
	 *
	 * @param db       The database to search.
	 * @param provides The provided dependency to search with.
	 * @param matches  The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 * 
	 * @see pmFindByProvides()
	 */
	PmStatus (*findByProvides)(PmDatabase *db, const char *provides,
							   PmMatches *matches);

	/**
	 * Searches for a package by the specified conflicting dependency.
	 *
	 * @param db        The database to search.
	 * @param conflicts The conflicting dependency to search with.
	 * @param matches   The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmFindByConflicts()
	 */
	PmStatus (*findByConflicts)(PmDatabase *db, const char *conflicts,
								PmMatches *matches);

	/**
	 * Returns all packages in the database.
	 *
	 * @param db      The database.
	 * @param matches The returned PmMatches structure.
	 *
	 * @return The status of the search.
	 *
	 * @see pmGetAllPackages()
	 */
	PmStatus (*getAllPackages)(PmDatabase *db, PmMatches *matches);

	/**
	 * Returns the first matched package.
	 *
	 * @param db      The database.
	 * @param matches The matches.
	 *
	 * @return The first matched package.
	 *
	 * @see pmFirstPackage()
	 */
	PmPackage *(*firstPackage)(PmDatabase *db, PmMatches *matches);

	/**
	 * Returns the next matched package.
	 *
	 * @param db      The database.
	 * @param matches The matches.
	 *
	 * @return The next matched package.
	 *
	 * @see pmNextPackage()
	 */
	PmPackage *(*nextPackage)(PmDatabase *db, PmMatches *matches);

	/**
	 * Destroys a PmMatches structure.
	 *
	 * @param matches The structure to destroy.
	 */
	PmStatus (*destroyMatches)(PmMatches *matches);

	/**
	 * Destroys any database-specific data in a package structure.
	 *
	 * @param data The data to destroy.
	 */
	void (*destroyPkgData)(void *data);

	/**
	 * Gets a list of files from a package in the database and adds
	 * them to the package.
	 *
	 * @param db  The database.
	 * @param pkg The package.
	 */
	void (*getFiles)(PmDatabase *db, PmPackage *pkg);

	/**
	 * Gets a list of required dependencies from a package in the
	 * database and adds them to the package.
	 *
	 * @param db  The database.
	 * @param pkg The package.
	 */
	void (*getRequiredDeps)(PmDatabase *db, PmPackage *pkg);

	/**
	 * Gets a list of provided dependencies from a package in the
	 * database and adds them to the package.
	 *
	 * @param db  The database.
	 * @param pkg The package.
	 */
	void (*getProvidedDeps)(PmDatabase *db, PmPackage *pkg);

	/**
	 * Gets the script with the specified type, event, and trigger name, if
	 * the script is a trigger.
	 *
	 * @param db      The database.
	 * @param package The package.
	 * @param type    The type of script.
	 * @param event   The event.
	 * @param trigger The name of a package for a trigger, @c NULL if the
	 *                specified script is not a trigger script.
	 *
	 * @return The script if found, or @c NULL if not found.
	 */
	char *(*getScript)(PmDatabase *db, PmPackage *package, PmScriptType type,
					   PmScriptEvent event, const char *trigger);
};


/**
 * @struct _PmPackageOps module.h libpackman/module.h
 * 
 * Package operations structure.
 *
 * These functions should only be used by libpackman!
 */
struct _PmPackageOps
{
	/**
	 * Opens the specified package.
	 *
	 * @param package  A pointer to the parent package structure.
	 * @param filename The filename to open.
	 *
	 * @return The status of the operation.
	 *
	 * @see pmPackageRead()
	 */
	PmStatus (*open)(PmPackage *package, const char *filename);

	/**
	 * Closes the specified package.
	 *
	 * @param package A pointer to the parent package structure.
	 * 
	 * @return The status of the operation.
	 */
	PmStatus (*close)(PmPackage *package);

	/**
	 * Determines if this module handles the specified file type.
	 *
	 * @param filename The filename of the package.
	 *
	 * @return 1 if this module supports the specified file type;
	 * @return 0 otherwise.
	 */
	int (*handlesFileType)(const char *filename);

	/**
	 * Extracts a single file from an archive.
	 *
	 * @param package     A pointer to the parent package structure.
	 * @param archivePath The path of the file in the archive.
	 * @param fsPath      The destination path on the filesystem.
	 *
	 * @return The status of the operation.
	 */
	PmStatus (*extractFile)(PmPackage *package, const char *archivePath,
							const char *fsPath);

	/**
	 * Extracts the entire archive.
	 *
	 * @param package  A pointer to the parent package structure.
	 *
	 * @return The status of the operation.
	 */
	PmStatus (*extractAll)(PmPackage *package);

	/**
	 * Gets the script with the specified type, event, and trigger name, if
	 * the script is a trigger.
	 *
	 * @param package A pointer to the parent package structure.
	 * @param type    The type of script.
	 * @param event   The event.
	 * @param trigger The name of a package for a trigger, @c NULL if the
	 *                specified script is not a trigger script.
	 *
	 * @return The script if found, or @c NULL if not found.
	 */
	char *(*getScript)(PmPackage *package, PmScriptType type,
					   PmScriptEvent event, const char *trigger);
};


/**
 * @struct _PmModule module.h libpackman/module.h
 * 
 * A package or database module.
 *
 * @see PmModuleType
 * @see PmPackageOps
 * @see PmDatabaseOps
 */
struct _PmModule
{
	PmModuleType type;             /**< The type of module.              */

	char *filename;                /**< The filename of the module.      */
	char *name;                    /**< The name of the module.          */

	void *handle;                  /**< The libtool handle.              */

	unsigned int refCount;         /**< The module's reference count.    */

	union
	{
		PmDatabaseOps *db;         /**< Database operations.             */
		PmPackageOps  *package;    /**< Package operations.              */

	} ops;                         /**< Operations.                      */
	
	PmModule *prev;                /**< The previous module in the list. */
	PmModule *next;                /**< The next module in the list.     */
};

/**
 * Registers a module.
 *
 * @param name The name of the module.
 * @param ops  The operations structure.
 * @param type The type of module.
 *
 * @return The module structure, or NULL on failure.
 */
PmModule *pmRegisterModule(const char *name, void *ops, PmModuleType type);

/**
 * Loads the module of the specified name and type.
 *
 * @param name The name of the module (without ".so")
 * @param type The type of module.
 *
 * @return A PmModule structure representing the module.
 *
 * @see pmUnloadModule()
 * @see pmGetModule()
 */
PmModule *pmLoadModule(const char *name, PmModuleType type);

/**
 * Unloads the specified module.
 *
 * @param module The module to unload.
 *
 * @see pmLoadModule()
 */
void pmUnloadModule(PmModule *module);

/**
 * Returns the specified module.
 *
 * The module will be automatically loaded if it's not already loaded.
 *
 * @param name The name of the module.
 * @param type The type of the module.
 *
 * @return A PmModule structure representing the module.
 *
 * @see pmLoadModule()
 */
PmModule *pmGetModule(const char *name, PmModuleType type);

/**
 * Notifies libpackman that the module is being used.
 *
 * This is used to increment the module's reference count.
 *
 * @param ptr The module.
 *
 * @see pmUnlinkModule()
 */
void pmLinkModule(PmModule **ptr);

/**
 * Notifies libpackman that the module is no longer being used.
 *
 * This is used to decrement the module's reference count.
 *
 * @param ptr The module.
 *
 * @see pmLinkModule()
 */
void pmUnlinkModule(PmModule **ptr);

/**
 * Returns the first module in the list.
 *
 * This will probably only be useful inside libpackman.
 *
 * @param type The type of module to retrieve.
 * 
 * @return The first module in the list.
 *
 * @see pmGetModule()
 */
PmModule *pmGetFirstModule(PmModuleType type);

/**
 * Unloads all modules.
 *
 * This should only be called when you are absolutely done with this
 * library.
 *
 * This will be automatically called by pmCleanup().
 *
 * @see pmCleanup()
 */
void pmCleanupModules(void);

/**
 * Cleans up the entire library.
 * 
 * This should only be called when you are absolutely done with this
 * library.
 *
 * This will call pmCleanupModules() automatically.
 *
 * @see pmCleanupModules()
 */
void pmCleanup(void);

#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_MODULE_H_ */
