/**
 * @file libpackman/package.h Package structure and support API
 *
 * $Id: package.h,v 1.26 2002/03/06 01:36:08 chipx86 Exp $
 *
 * @Copyright (C) 1999-2002 The GNUpdate Project.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */
#ifndef _LIBPACKMAN_PACKAGE_H_
#define _LIBPACKMAN_PACKAGE_H_

#include <time.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct _PmPackage PmPackage;   /**< PmPackage. */

#include <libpackman/types.h>
#include <libpackman/file.h>
#include <libpackman/downloadurl.h>
#include <libpackman/dependency.h>
#include <libpackman/module.h>
#include <libpackman/db.h>

/**
 * @struct _PmPackage package.h libpackman/package.h
 * 
 * A representation of a package.
 */
struct _PmPackage
{
	PmModule *module;                /**< The module.                        */

	PmAccessMode accessMode;         /**< The file access mode.              */

	PmDatabase *db;                  /**< The database this resides in.      */

	char *pkgFormat;                 /**< The package's format.              */
	char *osDistrib;                 /**< The OS distribution.               */

	char *filePath;                  /**< The file path of the package.      */
	
	char *name;                      /**< The package's name.                */
	char *version;                   /**< The package's version.             */
	char *release;                   /**< The package's release identifier.  */
	char *url;                       /**< The website of the package.        */
	char *group;                     /**< The categorical group the package
									  **  belongs to.                        */
	char *license;                   /**< The package's license.             */
	char *summary;                   /**< A brief summary of the package.    */
	char *description;               /**< A detailed description of the
									  **  package. */
	char *arch;                      /**< The supported architecture.        */
	char *branch;                    /**< The branch of the package.         */
	int fileSize;                    /**< The size of the package file.      */
	int installedSize;               /**< The total size of all installed
									  **  files.                             */

	PmDownloadUrl *downloadUrls;     /**< All URLs this package can be
									  **  downloaded from.                   */
	PmDownloadUrl *lastDownloadUrl;  /**< The last download URL in the list. */
	unsigned int downloadUrlCount;   /**< The number of download URLs.       */
	
	PmDependency *requirements;      /**< All required dependencies.         */
	PmDependency *lastRequirement;   /**< The last required dependency.      */
	unsigned int requirementCount;   /**< The number of requirements.        */

	PmDependency *provides;          /**< All provided dependencies.         */
	PmDependency *lastProvide;       /**< The last provided dependency.      */
	unsigned int provideCount;       /**< The number of provides.            */

	PmFile *files;                   /**< All stored files.                  */
	PmFile *lastFile;                /**< The last stored file.              */
	unsigned int fileCount;          /**< The number of files.               */

	PmPackage *prev;                 /**< Previous package in the list.      */
	PmPackage *next;                 /**< Next package in the list.          */

	char _depsVerified;              /**< All dependencies verified.         */
	unsigned int _curStep;           /**< The current step. Don't touch.     */
	unsigned int _totalSteps;        /**< Total steps. Don't touch.          */
	char _curOperation;              /**< Current operation. Don't touch.    */

	void *dbData;                    /**< Database-specific data.            */
};


/**************************************************************************/
/** @name PmPackage (De)allocation Functions                              */
/**************************************************************************/
/*@{*/

/**
 * Creates a new PmPackage structure.
 *
 * @return A new, empty PmPackage structure.
 *
 * @see pmDestroyPackage()
 */
PmPackage *pmNewPackage(void);

/**
 * Destroys a PmPackage structure.
 * 
 * @param package The structure to destroy.
 *
 * @see pmNewPackage()
 */
void pmDestroyPackage(PmPackage *package);

/*@}*/


/**************************************************************************/
/** @name Package Modification Functions                                  */
/**************************************************************************/
/*@{*/

/**
 * Adds a download URL to the package.
 *
 * @param pkg         The package to add the download URL to.
 * @param downloadUrl The download URL to add.
 *
 * @see PmDownloadUrl
 * @see pmFirstDownloadUrl()
 * @see pmPreviousDownloadUrl()
 * @see pmNextDownloadUrl()
 */
void pmPackageAddDownloadUrl(PmPackage *pkg, PmDownloadUrl *downloadUrl);

/**
 * Adds a required dependency to the package.
 *
 * @param pkg         The package to add the required dependency to.
 * @param requirement The required dependency to add.
 *
 * @see PmDependency
 * @see pmFirstRequirement()
 * @see pmPreviousRequirement()
 * @see pmNextRequirement()
 */
void pmPackageAddRequirement(PmPackage *pkg, PmDependency *requirement);

/**
 * Adds a provided dependency to the package.
 *
 * @param pkg     The package to add the provided dependency to.
 * @param provide The provided dependency to add.
 *
 * @see PmDependency
 * @see pmFirstProvide()
 * @see pmPreviousProvide()
 * @see pmNextProvide()
 */
void pmPackageAddProvide(PmPackage *pkg, PmDependency *provide);

/**
 * Adds a file to the package.
 *
 * @param pkg  The package to add the file to.
 * @param file The file to add.
 *
 * @see PmFile
 * @see pmFirstFile()
 * @see pmPreviousFile()
 * @see pmNextFile()
 */
void pmPackageAddFile(PmPackage *pkg, PmFile *file);

/*@}*/


/**************************************************************************/
/** @name Attribute Modification Functions                                */
/**************************************************************************/
/*@{*/

/**
 * Sets the package's associated module.
 *
 * This should really only be used by libpackman.
 * 
 * @param pkg    The package.
 * @param module The associated module.
 *
 * @see pmGetPackageModule()
 */
void pmSetPackageModule(PmPackage *pkg, PmModule *module);

/**
 * Sets the package's file access mode.
 *
 * @param pkg  The package.
 * @param mode The access mode.
 *
 * @see pmGetPackageAccessMode()
 */
void pmSetPackageAccessMode(PmPackage *pkg, PmAccessMode mode);

/**
 * Sets the package's file format.
 *
 * This should really only be used by libpackman.
 * 
 * @param pkg       The package.
 * @param pkgFormat The file format of the package.
 *
 * @see pmGetPackageFormat()
 */
void pmSetPackageFormat(PmPackage *pkg, const char *pkgFormat);

/**
 * Sets the package's supported OS distribution.
 *
 * This should really only be used by libpackman.
 *
 * @param pkg       The package.
 * @param osDistrib The supported OS distribution.
 *
 * @see pmGetPackageOsDistrib()
 */
void pmSetPackageOsDistrib(PmPackage *pkg, const char *osDistrib);

/**
 * Sets the package's file path.
 *
 * This should really only be used by libpackman.
 *
 * @param pkg      The package.
 * @param filePath The physical location of the package.
 *
 * @see pmGetPackageFilePath()
 */
void pmSetPackageFilePath(PmPackage *pkg, const char *filePath);

/**
 * Sets the package's name.
 *
 * @param pkg  The package.
 * @param name The package's name.
 *
 * @see pmGetPackageName()
 */
void pmSetPackageName(PmPackage *pkg, const char *name);

/**
 * Sets the package's version.
 *
 * @param pkg     The package.
 * @param version The package's version.
 *
 * @see pmGetPackageVersion()
 */
void pmSetPackageVersion(PmPackage *pkg, const char *version);

/**
 * Sets the package's release identifier.
 *
 * @param pkg     The package.
 * @param release The release identifier.
 *
 * @see pmGetPackageRelease()
 */
void pmSetPackageRelease(PmPackage *pkg, const char *release);

/**
 * Sets the package's website's URL.
 *
 * @param pkg The package.
 * @param url The website's URL.
 *
 * @see pmGetPackageUrl()
 */
void pmSetPackageUrl(PmPackage *pkg, const char *url);

/**
 * Sets the package's categorical group.
 *
 * @param pkg   The package.
 * @param group The package's group.
 *
 * @see pmGetPackageGroup()
 */
void pmSetPackageGroup(PmPackage *pkg, const char *group);

/**
 * Sets the package's license.
 *
 * @param pkg     The package.
 * @param license The license.
 *
 * @see pmGetPackageLicense()
 */
void pmSetPackageLicense(PmPackage *pkg, const char *license);

/**
 * Sets a brief summary of the specified package.
 *
 * @param pkg     The package.
 * @param summary The brief summary of the package.
 *
 * @see pmGetPackageSummary()
 */
void pmSetPackageSummary(PmPackage *pkg, const char *summary);

/**
 * Sets a detailed description of the specified package.
 *
 * @param pkg         The package.
 * @param description The description of the package.
 *
 * @see pmGetPackageDescription()
 */
void pmSetPackageDescription(PmPackage *pkg, const char *description);

/**
 * Sets the supported architecture of the specified package.
 *
 * Common architectures include (but are not limited to):
 *     - i386
 *     - i486
 *     - i586
 *     - i686
 *     - alpha
 *     - src
 *     - noarch
 *
 * @param pkg  The package.
 * @param arch The supported architecture.
 *
 * @see pmGetPackageArch()
 */
void pmSetPackageArch(PmPackage *pkg, const char *arch);

/**
 * Sets the release branch of the specified package.
 *
 * Common branches include (but are not limited to):
 *     - default
 *     - stable
 *     - unstable
 *     - cvs
 *
 * @param pkg    The package.
 * @param branch The release branch of the package.
 *
 * @see pmGetPackageBranch()
 */
void pmSetPackageBranch(PmPackage *pkg, const char *branch);

/**
 * Sets the file size of the package file.
 *
 * @param pkg      The package.
 * @param fileSize The file size.
 *
 * @see pmGetPackageFileSize()
 */
void pmSetPackageFileSize(PmPackage *pkg, unsigned int fileSize);

/**
 * Sets the total installed size of all files in the package.
 *
 * @param pkg           The package.
 * @param installedSize The total installed size of the package's contents.
 *
 * @see pmGetPackageInstalledSize()
 */
void pmSetPackageInstalledSize(PmPackage *pkg, unsigned int installedSize);

/**
 * Sets the previous package in the list.
 *
 * @param pkg  The current package.
 * @param prev The previous package.
 *
 * @see pmGetNextPackage()
 * @see pmGetPreviousPackage()
 * @see pmSetNextPackage()
 */
void pmSetPreviousPackage(PmPackage *pkg, PmPackage *prev);

/**
 * Sets the next package in the list.
 *
 * @param pkg  The current package.
 * @param next The next package.
 *
 * @see pmGetNextPackage()
 * @see pmGetPreviousPackage()
 * @see pmSetPreviousPackage()
 */
void pmSetNextPackage(PmPackage *pkg, PmPackage *next);

/*@}*/


/**************************************************************************/
/** @name Attribute Retrieval Functions                                   */
/**************************************************************************/
/*@{*/

/**
 * Returns the package's module.
 *
 * @param pkg The package.
 *
 * @return The package's module.
 *
 * @see pmSetPackageModule()
 */
PmModule *pmGetPackageModule(PmPackage *pkg);

/**
 * Returns the package's file access mode.
 *
 * @param pkg The package.
 *
 * @return The file access mode, or PM_MODE_ERROR if @a pkg is NULL.
 * 
 * @see pmSetPackageAccessMode()
 */
PmAccessMode pmGetPackageAccessMode(PmPackage *pkg);

/**
 * Returns the package's file format.
 *
 * @param pkg The package.
 *
 * @return The package's file format.
 *
 * @see pmSetPackageFormat()
 */
const char *pmGetPackageFormat(PmPackage *pkg);

/**
 * Returns the package's supported OS distribution.
 *
 * @param pkg The package.
 *
 * @return The package's supported OS distribution.
 *
 * @see pmSetPackageOsDistrib()
 */
const char *pmGetPackageOsDistrib(PmPackage *pkg);

/**
 * Returns the package's file path.
 *
 * @param pkg The package.
 * 
 * @return The physical location of the package.
 *
 * @see pmSetPackageFilePath()
 */
const char *pmGetPackageFilePath(PmPackage *pkg);

/**
 * Returns the package's name.
 *
 * @param pkg The package.
 *
 * @return The package's name.
 *
 * @see pmSetPackageName()
 */
const char *pmGetPackageName(PmPackage *pkg);

/**
 * Returns the package's version.
 * 
 * @param pkg The package.
 *
 * @return The package's version.
 *
 * @see pmSetPackageVersion()
 */
const char *pmGetPackageVersion(PmPackage *pkg);

/**
 * Returns the package's release identifier.
 *
 * @param pkg The package.
 *
 * @return The package's release identifier.
 *
 * @see pmSetPackageRelease()
 */
const char *pmGetPackageRelease(PmPackage *pkg);

/**
 * Returns the package's website's URL.
 *
 * @param pkg The package.
 *
 * @return The package's website's URL.
 *
 * @see pmSetPackageUrl()
 */
const char *pmGetPackageUrl(PmPackage *pkg);

/**
 * Returns the package's categorical group.
 *
 * @param pkg The package.
 *
 * @return The package's group.
 *
 * @see pmSetPackageGroup()
 */
const char *pmGetPackageGroup(PmPackage *pkg);

/**
 * Returns the package's license.
 *
 * @param pkg The package.
 *
 * @return The package's license.
 *
 * @see pmSetPackageLicense()
 */
const char *pmGetPackageLicense(PmPackage *pkg);

/**
 * Returns a brief summary of the specified package.
 *
 * @param pkg The package.
 *
 * @return A brief summary.
 *
 * @see pmSetPackageSummary()
 */
const char *pmGetPackageSummary(PmPackage *pkg);

/**
 * Returns a detailed description of the specified package.
 *
 * @param pkg The package.
 * 
 * @return A detailed description.
 *
 * @see pmSetPackageDescription()
 */
const char *pmGetPackageDescription(PmPackage *pkg);

/**
 * Returns the supported architecture of the specified package.
 *
 * @param pkg The package.
 *
 * @return The supported architecture.
 *
 * @see pmSetPackageArch()
 */
const char *pmGetPackageArch(PmPackage *pkg);

/**
 * Returns the package's release branch.
 *
 * @param pkg The package.
 *
 * @return The package's release branch.
 *
 * @see pmSetPackageBranch()
 */
const char *pmGetPackageBranch(PmPackage *pkg);

/**
 * Returns the file size of the specified package.
 * 
 * @param pkg The package.
 *
 * @return The file size of the package.
 *
 * @see pmSetPackageFileSize()
 */
unsigned int pmGetPackageFileSize(PmPackage *pkg);

/**
 * Returns the total size of all installed files in the package.
 *
 * @param pkg The package.
 *
 * @return The total size of all installed files.
 *
 * @see pmSetPackageInstalledSize()
 */
unsigned int pmGetPackageInstalledSize(PmPackage *pkg);

/**
 * Returns the number of download URLs in the specified package.
 *
 * @param pkg The package.
 *
 * @return The number of download URLs.
 *
 * @see pmPackageAddDownloadUrl()
 */
unsigned int pmGetPackageDownloadUrlCount(PmPackage *pkg);

/**
 * Returns the number of required dependencies in the specified package.
 *
 * @param pkg The package.
 *
 * @return The number of required dependencies.
 *
 * @see pmPackageAddRequirement()
 */
unsigned int pmGetPackageRequirementCount(PmPackage *pkg);

/**
 * Returns the number of provided dependencies in the specified package.
 *
 * @param pkg The package.
 *
 * @return The number of provided dependencies.
 *
 * @see pmPackageAddProvide()
 */
unsigned int pmGetPackageProvideCount(PmPackage *pkg);

/**
 * Returns the number of files in the specified package.
 *
 * @param pkg The package.
 *
 * @return The number of files.
 *
 * @see pmPackageAddFile()
 */
unsigned int pmGetPackageFileCount(PmPackage *pkg);

/**
 * Returns the previous package in the list.
 *
 * @param pkg The current package.
 * 
 * @return The previous package in the list.
 *
 * @see pmGetNextPackage()
 * @see pmSetNextPackage()
 * @see pmSetPreviousPackage()
 */
PmPackage *pmGetPreviousPackage(PmPackage *pkg);

/**
 * Returns the next package in the list.
 *
 * @param pkg The current package.
 *
 * @return The next package in the list.
 *
 * @see pmGetPreviousPackage()
 * @see pmSetNextPackage()
 * @see pmSetPreviousPackage()
 */
PmPackage *pmGetNextPackage(PmPackage *pkg);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif /* _LIBPACKMAN_PACKAGE_H_ */
